"""
Wisent - Client library for interacting with the Wisent backend services.
"""

from wisent.client import WisentClient
from wisent.version import __version__

__all__ = ["WisentClient", "__version__"]
